<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\COZ\Services\CelulaprodServicoFilialTipopratService;

class CelulaprodServicoFilialTipopratController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $CelulaprodServicoFilialTipopratService;
    protected $dataSourceName = "celulaprod_servico_filial_tipoprat";
    protected $parameterBagColumns = array("CDFILIAGREGA", "NRORG");
    protected $environment;
    

    public function __construct($crudService, $parameterBag, CelulaprodServicoFilialTipopratService $celulaprodServicoFilialTipopratService, $environment)
    {
        parent::__construct($crudService, $parameterBag);
        $this->celulaprodServicoFilialTipopratService = $celulaprodServicoFilialTipopratService;
        $this->environment = $environment;
    }

    public function excluiCelulaProducao (Request\Row $request, Response $response){
        try {
            $params = $request->getRow();
            if ($params['DTPRODUCAO']) {
                $this->celulaprodServicoFilialTipopratService->excluiCelulaProducao(
                    $params['NRORG'], $params['DTPRODUCAO'],
                    $params['NRDIASEMANA'],
                    $params['CDFILIAGREGA'], $params['CDSERVFILI'],
                    $params['CENTRALTPPRAT']
                );
            } else {
                $this->celulaprodServicoFilialTipopratService->excluiCelulaProducaoPorServico(
                    $params['NRORG'], $params['CDFILIAGREGA'],
                    $params['CDSERVFILI']
                );
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }

    public function salvaCelulaProducao (Request\Row $request, Response $response){
        try {
            $params = $request->getRow();
            $dataSourceName = "celulaprod";
            $retorno =array('EXISTE' => 0, 'INSERE' => 0);
            $rows = array();
            $currentTime = $this->celulaprodServicoFilialTipopratService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
             
            $cdfiliagrega = $params['CDFILIAGREGA'];
            $dtproducao = $params['DTPRODUCAO'];
            $cdcentral = $params['CDCENTRAL'];
            $nrorg = $params['NRORG'];
            $params = $this->carregaDados($params);
            
            $cdservfili = $params['CDSERVFILI'];

            if (!$cdservfili) {
                throw new \Exception('Operação Bloqueada! Não há serviço transportado cadastrado.');
            }
            
            $nrdiasemana = $params['NRDIASEMANA'];
            $cdtipoprat = $params['CDTIPOPRAT'];
            if (!$cdtipoprat) {
                throw new \Exception('Operação Bloqueada! Não há tipo de prato cadastrado.');
            }

            $conditions = [
                array(
                    "columnName" => 'DTPRODUCAO',
                    "operator"   => '=',
                    "value"      => $dtproducao
                ),
                array(
                    "columnName" => 'CDFILIAGREGA',
                    "operator"   => '=',
                    "value"      => $cdfiliagrega
                ),
                array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      => $this->celulaprodServicoFilialTipopratService->getEnvironment()->getNrOrgTrab()
                )
            ];
            $rowsExistentes = $this->crudService->getFilteredDataSet("celulaprod_servico_filial_tipoprat", $conditions);       
            $this->verificaServicos($cdservfili, $cdcentral);
            foreach ($cdservfili as $servico) {
                foreach ($cdtipoprat as $tipoprato) {
                    foreach ($nrdiasemana as $diaSemana) {
                        $array = array(
                            'NRORG' => $nrorg,
                            'CDCENTRAL' => $cdcentral,
                            'DTPRODUCAO' => $dtproducao,
                            'NRDIASEMANA' => $diaSemana,
                            'CDTIPOPRAT' => $tipoprato,
                            'CDFILIAGREGA' => $cdfiliagrega,
                            'CDSERVFILI' => $servico,
                            'DTULTATU' => $currentTime,
                            'NRORGULTATU' => $this->celulaprodServicoFilialTipopratService->getEnvironment()->getNrOrgTrab(),
                            'CDOPERULTATU' => $this->celulaprodServicoFilialTipopratService->getEnvironment()->getCdOperador(),
                            'DTINCLUSAO' => $currentTime,
                            'NRORGINCLUSAO' => $this->celulaprodServicoFilialTipopratService->getEnvironment()->getNrOrgTrab(),
                            'CDOPERINCLUSAO' => $this->celulaprodServicoFilialTipopratService->getEnvironment()->getCdOperador(),
                            '__is_new' => true
                        );

                        array_push($rows, $array);
                    }
                }
            }

            if (count($rowsExistentes->getRows()) > 0) {
                foreach ($rows as $chave => $row) {
                    foreach ($rowsExistentes->getRows() as $existentes) {
                        if ($existentes['CDTIPOPRAT'] == $row['CDTIPOPRAT'] and
                            $existentes['CDSERVFILI'] == $row['CDSERVFILI']
                            and $existentes['NRDIASEMANA'] == $row['NRDIASEMANA']
                        ) {
                            unset($rows[$chave]);
                            $retorno['EXISTE'] = 1;
                            continue 2;
                        }
                    }
                }
            }

            if(count($rows) > 0){
                $dataSet = new DataSet($dataSourceName, $rows);
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rowsToSave);
				
				$retorno['INSERE'] = 1; 
            }
            
            $response->addDataset(new DataSet('RETORNO', $retorno));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function carregaDados($params){
        if (!$params['CDSERVFILI']) {
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('servicos_por_grupo_e_agregadas');
            $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDOPERADOR', 'CDGRUPSERV');
            $filterCriteria->addCondition('NRORG', $params['NRORG']);
            $filterCriteria->addCondition('CDFILIAL', $params['CDFILIAGREGA']);
            $filterCriteria->addCondition('CDOPERADOR', $params['CDOPERADOR']);
            $filterCriteria->setDataSourceName('servicos_por_grupo_e_agregadas');

            if (!$params['CDSERVFILI'] AND !$params['CDGRUPSERV']) {
                $filterCriteria->addCondition('CDGRUPSERV', "T");
            } else if (!$params['CDSERVFILI'] AND $params['CDGRUPSERV']) {
                $filterCriteria->addCondition('CDGRUPSERV', $params['CDGRUPSERV']);
            }

            $response = new Response();
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
            $params['CDSERVFILI'] = array_column($response->getDataSets()[0]->getRows(), "CDSERVFILI");
        }

        if (!$params['NRDIASEMANA']) {
            $params['NRDIASEMANA'] = array("1", "2", "3", "4", "5", "6", "7", "8", "9");
        }

        if (!$params['CDTIPOPRAT']) {
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('tipoprat');
            $filterCriteria->addCondition('NRORG', $params['NRORG']);

            $response = new Response();
            $this->crudService->find($response, $filterCriteria);
            $params['CDTIPOPRAT'] = array_column($response->getDataSets()[0]->getRows(), "CDTIPOPRAT");
        }

        return $params;
    }

    public function retornaAgregadaCelulaProd(Request\Filter $request, Response $response){
        $this->dataSourceName = "agregada_celulaProd";
        $this->find($request, $response);
    }
    
    public function retornaCelulaProducao(Request\Row $request, Response $response){
        $row = $request->getRow();

        $conditions = [
                array(
                    "columnName" => 'CDFILIAGREGA',
                    "operator"   => '=',
                    "value"      => $row['CDFILIAGREGA']
                ),
                array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      => $row['NRORG']
                )
        ];
        
        
        $dataSet = $this->crudService->getFilteredDataSet("celulaprod_servico_filial_tipoprat", $conditions);  
        $rows = $dataSet->getRows();
        
        //filtra por diferentes serviços
        $dadosDistinct = [];
        
        if(count($rows) != 0){
            foreach ($rows as $linha) {
               $filtered = [];
               foreach($dadosDistinct as $linhafilter){
                   if($linha['CDSERVFILI'] == $linhafilter['CDSERVFILI']) $filtered = $linha['CDSERVFILI'];
               }
               if(empty($filtered))array_push($dadosDistinct,$linha);
            }
        }
        $response->addDataset(new DataSet('celulas', $dadosDistinct));
        
    }
    
    private function verificaServicos($servicos, $cdcentral){
        $dataSourceName = 'servicos_por_grupo_e_agregadas';
    	$conditions = array(
    	    array(
    	    	"columnName" => "CDFILIAL",
    			"operator" => "=",
    			"value" => $cdcentral
        	    ),
        	array(
    	    	"columnName" => "CDGRUPSERV",
    			"operator" => "=",
    			"value" => 'T'
        	    ),
        	array(
    	    	"columnName" => "CDOPERADOR",
    			"operator" => "=",
    			"value" => $this->environment->getCdOperador()
        	    ),
        	array(
    	    	"columnName" => "NRORG",
    			"operator" => "=",
    			"value" => $this->environment->getNrOrgTrab()
        	    ),
    	);
    	
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $rows = $dataSet->getRows();
        $servicosTransp = array_map(function($row){
            return $row['CDSERVFILI'];
        }, $rows);
        foreach($servicos as $serv){
            if(array_search($serv, $servicosTransp) === false) throw new \Exception("Serviço(s) não é transportado!");
        }
    }
}