<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\COZ\Services\Pla10500Repository;

class Pla01600 extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $qr2Strategy;
    protected $novoCodigo;
    protected $lista;
    protected $nrlistaFilialServico;
    protected $nrlistaFilialSetor;
    protected $pla10500Repository;
    protected $birtStrategy;

    public function __construct(
        ReportService $reportService, $entityManager, $qr2Strategy, $novoCodigo,
        $lista, Pla10500Repository $pla10500Repository, $birtStrategy
    ) {
        $this->entityManager = $entityManager;
        $this->qr2Strategy = $qr2Strategy;
        $this->novoCodigo = $novoCodigo;
        $this->lista = $lista;
        $this->pla10500Repository = $pla10500Repository;
        $this->birtStrategy = $birtStrategy;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'req01600' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01601' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01602' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01604' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'mPLA00703' =>
            array (
                'strategy' => 'QR2',
                'parameterMapping' =>
                    array (
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTA' => 'P_NRLISTA',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'NRORG' => 'P_NRORG'
                    )
            ),
        'req01800' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'NRORG' => 'P_NRORG',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'CDPICTPROD' => 'P_CDPICTPROD',
                        'CONSDIST' => 'P_CONSDIST',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'ORDENA' => 'P_ORDENA'
                    )
            ),
            'REQ01801' => 
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' => 
                        array (
                            'NRPLANPROD'       => 'P_NRPLANPROD',
                            'DSPLANEJPRODUCAO' => 'P_DSPLANPROD',
                            'NRLISTASERVICO'   => 'P_LISTASERVICO',
                            'CDARVPRODINI'     => 'P_PRODINI',
                            'CDARVPRODFIN'     => 'P_PRODFIN',
                            'NRORG'            => 'P_NRORG',
                            'CDCENTRAL'        => 'P_CDCENTRAL',
                            'DTINICIAL'        => 'P_DTINICIAL',
                            'DTFINAL'          => 'P_DTFINAL',
                            'CONSCARD'         => 'P_CONSCARD',
                            'NRCONSPLAN'       => 'P_NRCONSPLAN',
                            'CONSFIXO'         => 'P_CONSFIXO',
                            'CONSPERCAP'       => 'P_CONSPERCAP',
                            'CONSDIST'         => 'P_CONSDIST',
                            'NMCENTRAL'        => 'P_NMCENTRAL',
                            'CDOPERADOR'       => 'P_CDOPERADOR',
                            'NMOPERADOR'       => 'P_NMOPERADOR',
                            'CDPICTPROD'       => 'P_CDPICTPROD',
                            'CDPICTPRAT'       => 'P_CDPICTPRAT',
                            'IDSUPPRAT'        => 'P_IDSUPPRAT'
                        )
                ),
            'req01802' => 
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' => 
                        array (
                            'CDOPERADOR'       => 'P_CDOPERADOR',
                            'NMOPERADOR'       => 'P_NMOPERADOR',
                            'NRPLANPROD'       => 'P_NRPLANPROD',
                            'DSPLANEJPRODUCAO' => 'P_DSPLANPROD',
                            'IDARREDOND'       => 'P_ARREDONDA',
                            'CONSCARD'         => 'P_CONSCARD',
                            'NRLISTASERVICO'   => 'P_LISTASERVICO',
                            'NRORG'            => 'P_NRORG',
                            'DTINICIAL'        => 'P_DTINICIAL',
                            'DTFINAL'          => 'P_DTFINAL',
                            'CDCENTRAL'        => 'P_CDCENTRAL',
                            'NRCONSPLAN'       => 'P_NRCONSPLAN',
                            'CONSFIXO'         => 'P_CONSFIXO',
                            'CONSPERCAP'       => 'P_CONSPERCAP',
                            'CONSDIST'         => 'P_CONSDIST',
                            'CDARVPRODINI'     => 'P_PRODINI',
                            'CDARVPRODFIN'     => 'P_PRODFIN',
                            'NRLISTASETOR'     => 'P_CDSETOR',
                            'ORDENA'           => 'P_ORDENA',
                            'CDPICTPROD'       => 'P_CDPICTPROD'
                        )
                )
            
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $row['NRORG'] = $this->pla10500Repository->environment->getNrorgTrab();
        
        if($row['CD_REPORT_NAME'] == 'mPLA00703'){
                $row['NRLISTA'] = $this->montaListaFilialServico(
                $row['NRORG'], $row['LISTASERVICO']
            );   
        }else $row['NRLISTASERVICO'] = $this->lista->geraListaServico($row['LISTASERVICO'], $row['NRORG']);
       
        $dtInicial = $row['DTINICIAL'];
        $dtFinal = $row['DTFINAL'];
        
        $consfixo = $row['CONSFIXO'];
        $conspercap = $row['CONSPERCAP'];
        $distribuicao = $row['CONSDIST'];
        
        $listaServ = array();

        if (!$row['NRPLANPROD']) {
			$lista = $row['LISTASERVICO'];
			if (substr($lista, -1) == '_') $row['LISTASERVICO'] = substr($row['LISTASERVICO'], 0, -1);
			
            $servicos = explode('_', $row['LISTASERVICO']);
            foreach ($servicos as $servico) {
                $listaServ[] = array('SERVICO' => $servico);
            }
        } else {
            $connection = $this->entityManager->getConnection();
            $query
                = "SELECT DISTINCT CDSERVFILIAGREG, CDFILIAGREG, CDFILIAGREG || CDSERVFILIAGREG AS CONCAT
                            FROM PLANPRODFILIAGREGA
                           WHERE NRPLANPROD = :P_NRPLANPROD OR :P_NRPLANPROD IS NULL";
            $resultado = $connection->fetchAll(
                $query, array("P_NRPLANPROD" => $row['NRPLANPROD'])
            );
            
            $listaServ = null;
            if (!empty($resultado)) {
                foreach ($resultado as $filiServ) {
                    if (strpos($row['LISTASERVICO'], $filiServ['CONCAT']) >= 0) {
                        $listaServ[] = array('SERVICO' => $filiServ['CONCAT']);
                    } 
                }
            }
        }
        
        if(isset($listaServ)){
            $consplan = $this->pla10500Repository->consplanGeral(
                array(
                    'NRORG' => $row['NRORG'],
                    'DTCARDAINI' => $dtInicial,
                    'DTCARDAFIN' => $dtFinal,
                    'CDOPERADOR' => $row['CDOPERADOR'],
                    'SERVICOFILIAL' => $listaServ
                ),
                $distribuicao . $consfixo . $conspercap,
                $row['IDARREDOND']
            );
            
        }else $consplan['P_NRCONSPLAN'] = 'xxxxxxxxxx';
        
        $row['NRCONSPLAN'] = $consplan['P_NRCONSPLAN'];
        
        $this->pla10500Repository->nrconsplan = $consplan['P_NRCONSPLAN'];
        $row['NRLISTASETOR'] = $this->montaListaFilialServico($row['NRORG'], $row['LISTASETOR']);
        
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

    }

    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $row = $request->getRow();
        if($row['CD_REPORT_NAME'] == 'mPLA00703'){
            $this->deleteLista($this->nrlistaFilialServico, $row['NRORG']);
            $this->pla10500Repository->deletaConsplan();
        }
    }

    public function montaListaFilialServico($nrorg, $filiaisServicos){
        $filiaisServicos = rtrim($filiaisServicos, '_');
        $arrayFiliaisServicos = explode('_', $filiaisServicos);
        
        if ($arrayFiliaisServicos[0]) {
            $this->nrlistaFilialServico = $this->novoCodigo->getCodeByTableName(
                "LISTA", $nrorg
            );
            foreach ($arrayFiliaisServicos as $filServ) {
                $FilialServicoParametros = array(
                    "NRORG" => $nrorg,
                    "NRLISTA" => $this->nrlistaFilialServico,
                    "CDLISTA" => $filServ
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '',
                    "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '',
                    "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '',
                    "DTULTATUPRFIL" => ''
                , "__is_new" => true
                );
                $this->saveLista($FilialServicoParametros);
            }
        } else {
            $this->nrlistaFilialServico = 0;
        }

        return $this->nrlistaFilialServico;
    }

    public function montaListaFilialSetor($nrorg, $filiaisServicos){
        $filiaisSetor = rtrim($filiaisServicos, '_');
        $arrayFiliaisSetor = explode('_', $filiaisSetor);
        if ($arrayFiliaisSetor[0]) {
            $this->nrlistaFilialSetor = $this->novoCodigo->getCodeByTableName(
                "LISTA", $nrorg
            );
            foreach ($arrayFiliaisSetor as $setor) {
                $FilialSetorParametros = array(
                    "NRORG" => $nrorg, "NRLISTA" => $this->nrlistaFilialSetor,
                    "CDLISTA" => $setor
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '',
                    "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '', "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '', "DTULTATUPRFIL" => ''
                , "__is_new" => true
                );
                $this->saveLista($FilialSetorParametros);
            }
        } else {
            $this->nrlistaFilialSetor = 0;
        }
        return $this->nrlistaFilialSetor;
    }

    public function saveLista($param){
        $request = new DTO\Request\Row($param, "save", "/dummy", " ");
        $response = new DTO\Response();
        $this->lista->save($request, $response);
    }

    public function deleteLista($nrLista, $nrorg=1){
        $connection = $this->entityManager->getConnection();
        $query="DELETE FROM LISTA WHERE NRLISTA = :NRLISTA AND NRORG=:NRORG";
        $connection->executeQuery($query, array("NRLISTA"=>$nrLista, "NRORG"=>$nrorg));
    }
}