<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\COZ\Services\PrevisaoConsumoCentralService;

class PrevisaoConsumoCentralController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $previsaoConsumoCentralService;
    protected $requisicaoPlanejamentoProducaoService;
    protected $dataSourceName = "previsaoconsumocentral";
    protected $parameterBagColumns = array(
    	"NRORG"
	);
    protected $pla08200Service;

	public function __construct(
        $crudService, $parameterBag, $previsaoConsumoCentralService,
        $requisicaoPlanejamentoProducaoService, $pla08200Service
    ) {
        $this->previsaoConsumoCentralService = $previsaoConsumoCentralService;
        $this->requisicaoPlanejamentoProducaoService = $requisicaoPlanejamentoProducaoService;
        $this->pla08200Service = $pla08200Service;
    	parent::__construct($crudService, $parameterBag);
    }

    public function calculaPrevisao(DTO\Request\Row $request, DTO\Response $response) {
        try {
            
            $rows = $request->getRow();
            $nrorg = $rows['NRORG'];
            $cdcentral = $rows['CDCENTRAL'];
            
            $paramsCentral = $this->retornaParamsplaneja($cdcentral)[0];
            
            if($paramsCentral['CDSERVRETPME'] != null && $paramsCentral['CDTPRETIPADR'] != null){
                
                $rows['CDGRUPFILI'] = ($rows['CDGRUPFILI'] === '' ? 'T' : $rows['CDGRUPFILI']);
                $rows["LISTAFILIAL"]   = $this->pla08200Service->retornaFiliais((array)$rows);
                $rows["LISTAGRUPSERV"] = $this->pla08200Service->retornaGrupServ((array)$rows);
                $rows['SERVICOFILIAL'] = $this->pla08200Service->retornaServico((array)$rows);
                
                $dtinicial = $rows['DTINICIAL'];
                $dtfinal   = $rows['DTFINAL'];
                $cdprodini = $rows['CDPRODINI'];
                $cdprodfin = $rows['CDPRODFIN'];
                $produtos  = $rows['IDPRODUTOS'];
                
                $nrlista = $this->requisicaoPlanejamentoProducaoService->geraLista($nrorg, $rows['SERVICOFILIAL'], '_');
                $this->previsaoConsumoCentralService->procedurePrevConsListaPlanprod($cdcentral, $nrlista, $dtinicial, $dtfinal, $cdprodini, $cdprodfin, $produtos);
                $this->requisicaoPlanejamentoProducaoService->deletaLista($nrorg, $nrlista);
            
                
            }else $response->addDataSet(new DataSet('paramsCentral', $paramsCentral));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function retornaParamsplaneja($cdcentral){
        $conditions = array(
        	array(
    	    	"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdcentral
        	    )
        	);
        	
		$dataSet = $this->crudService->getFilteredDataSet('paramplaneja', $conditions);
        $rows = $dataSet->getRows();
        return $rows;
    }
}